#!/bin/sh
#
# 5698-PDO
# Licensed Materials - Property of IBM
# 
# (C) Copyright IBM Corp. 2000,2008. All Rights Reserved.
# 
# US Government Users Restricted Rights - Use, duplication or
# disclosure restricted by GSA ADP Schedule Contract with
# IBM Corp.
#
OSS_i=/opt/pdos/
LPMNISLIB=${OSS_i}/lib/liblpmnis.so

# exit status values from lpmcfg.h.
pdolp_cfg_pam_ok=0
pdolp_cfg_platform_error=1
pdolp_cfg_error_update_pam=2
pdolp_cfg_error_unconfig_methods=3
pdolp_cfg_error_unconfig_user=4
pdolp_cfg_error_config_methods=5
pdolp_cfg_error_config_user=6
pdolp_cfg_error_swap_user=7
pdolp_cfg_error_update_conf=8
pdolp_cfg_error_nis_no_makefile=9
pdolp_cfg_error_nis_no_shadow=10
pdolp_cfg_error_nis_makefile=11
pdolp_cfg_error_nis_make=12
pdolp_usage_config=13

# Define values from the lpmnis.h
LPM_NIS_SECS=0
LPM_NIS_DAYS=1
LPM_NIS_WEEK=7
LPM_NIS_MAPNAME=passwdchg
x86_linux_2_gran=${LPM_NIS_DAYS}
s390_linux_2_gran=${LPM_NIS_DAYS}
ppc_linux_2_gran=${LPM_NIS_DAYS}
sparc_solaris_2_gran=${LPM_NIS_DAYS}
rios_aix_4_gran=${LPM_NIS_SECS}
hp9000_ux_10_gran=${LPM_NIS_WEEK}
LPM_EXT_LIBBASE=/usr/lib/liblpmext
LIBNAME_SUFF=.so
######################################################################
# This is all PRE phase code, common on all platforms.
######################################################################
#
# Exit/error status values must have been set from ../cli/lpmcfg.h
# THE FOLLOWING variables should be grabbed from the header lpmnis.h
#
# LPM_NIS_MAPNAME
# ${CONTEXT}_gran
#
# Uncomment the following line to get debugging info
#
#debug LEVELS
#debug=1
#debug_lil=1

YPDIR=/var/yp
#
# EXIT now if the Makefile doesen't exist!
#
if [ ! -f ${YPDIR}/Makefile ]; then
	exit ${pdolp_cfg_error_nis_no_makefile}
fi

#
# process the makefile using nawk, add directive to generate
# the map of password change dates.
#
if [ $debug ]; then
    echo "Make a copy of ${YPDIR}/Makefile."
fi
cp ${YPDIR}/Makefile ${YPDIR}/Makefile.bak.pdos
MAKE_TARGET=${LPM_NIS_MAPNAME}
######################################################################
# END PRE phase code.
######################################################################
#
# Platform specific code to prepare the yp makefile to support PDOS
# by generating a new map with password change times
#
awk 'BEGIN { deball=1000; deblil=100; shadow_name=""; \
    already1=0; already2=0; already3=0; merge=0; rc=0 };
{
    mapname_dep=mapname ":";
    mapname_byname=mapname ".byname";
    if (! ($0 ~ /^[a-zA-Z0-9_]/)) {
	print $0;
    } else if ($1 ~ /^SHADOW/) {
        shadow_name=$3
        if (deblil) {
            printf("# Grab the value of the shadow, %s.\n",shadow_name);
        }
        print $0
    } else if ($1 ~ /^all:/) {
        # This clause is to add the dependency for the PDOS map
        # to the list that is built by "all:"
        if (deblil) {
            printf("# MODIFY all: line to specify %s.\n",mapname);
        }
	first = 1;
	do {
	    if (first != 1) { print $0; getline; }
            comment=""; line="";
            pos = index($0,"#");
	    if (pos > 0) {
                # There is a comment here strip it
	        line=substr($0,1,(pos-1));
		comment=substr($0,pos);
            } else {
	        line=$0;
            }
            if (deblil) {
                printf("# Check line %s.\n",$0);
            }
            if (line ~ mapname) {
                if (deblil) {
                    printf("# Entry for %s already exists.\n",mapname);
                }
                already1 = 1;
            }
            first=0
        } while ((line ~ /\\$/) && !(already1 == 1))
        # Now, add ourselves to the end of the line, if we are NOT there!
        if ((line ~ mapname) || (already1 == 1)) {
            already1=1;
            if (deblil) {
                printf("# Entry for %s already (%d) exists.\n",
                    mapname,already1);
            }
            print $0
        } else {
            printf("%s %s %s\n", line, mapname, comment);
            already1 = 1;
        }
    } else if ($1 ~ mapname_dep) {
        already2 = 1;
        if (deblil) {
            printf("# Additional configuration ignored, %s.\n",mapname);
        }
        print $0
    } else if ($1 ~ /^passwd:/) {
        #
        # Add the dependency for the map, if not already done
        #
        if (already2 != 1) {
            printf("%s: \t%s\n",mapname,mapname_byname);
            already2=1;
        } else {
            printf("# Already done at least once!\n");
        }
        print $0
    } else if ($1 ~ mapname_byname) {
        already3 = 1;
        if (deblil) {
            printf("# Additional configuration is ignored, %s,%s.\n",
                mapname_byname,$1);
        }
        print $0
    } else if ($2 ~ /MERGE_PASSWD/) {
        if ((already3 != 1) && (shadow_name != "") ) {
            if (deblil) {
                printf("# Set dependence on system file, %s.\n",shadow_name);
            }
            #
            # now, output the make directive to make the map
            #
            printf("%s: $(SHADOW) $(YPDIR)/Makefile\n",mapname_byname);
            printf("\t@$(AWK) %cBEGIN { FS=\":\"; OFS=\":\" };",39);
            printf("/^[a-zA-Z0-9_]/ { printf(\"%cs\t%cd:%cd\\n\",$$1,gran,$$3); }%c \\\n",37,37,37,39);
            printf("\t\"gran=%d\" $(SHADOW) > /tmp/%s.in \n",gran,mapname);
            printf("\t$(DBLOAD) -r -i $(SHADOW) -o $(YPMAPDIR)/$@ /tmp/%s.in $@\n",mapname_byname,mapname);
            printf("\t-@$(NOPUSH) || $(YPPUSH) -d $(DOMAIN) $@\n\n");
            already3=1; merge=1;
        } else if (already3 != 1) {
            printf("# Cannot configure NIS for PDOS if no shadow file!\n");
            rc=noshadow;
        }
        print $0
    } else if ($1 ~ /^passwd.byname:/) {
        if ((already3 != 1) && (shadow_name != "") ) {
            if (deblil) {
                printf("# Set dependence on system file, %s.\n",shadow_name);
            }
            #
            # now, output the make directive to make the map
            #
            printf("%s: $(SHADOW) $(YPDIR)/Makefile\n",mapname_byname);
            printf("\t(nawk %cBEGIN { FS=\":\"; OFS=\":\" };",39);
            printf("/^[a-zA-Z0-9_]/ { printf(\"%cs\t%cd:%cd\\n\",$$1,gran,$$3); }%c \\\n",37,37,37,39);
            printf("\t\"gran=%d\" $(SHADOW) > /tmp/%s.in \n",gran,mapname);
            printf("\t@$(DBLOAD) -r -i $(SHADOW) -o $(YPMAPDIR)/$@ /tmp/%s.in $@\n",mapname_byname,mapname);
            printf("\t-@$(NOPUSH) || $(YPPUSH) -d $(DOMAIN) $@\n\n");
            already3=1
        } else if (already3 != 1) {
            printf("# Cannot configure NIS for PDOS if no shadow file!\n");
        }
        print $0
    } else {
        print $0
    }
} END {
    if (rc == 0) {
	if ((already1 == 0) || (already2 == 0) || (already3 == 0)) {
            if (deb) {
                printf("a1 %d a2 %d a3 %d.\n",already1,already2,already3);
            }
            rc=badmakefile;        
        }
    }
    outfile="/tmp/cfg.out." pid; print rc > outfile;
}'  "noshadow=${pdolp_cfg_error_nis_no_shadow}" \
    "badmakefile=${pdolp_cfg_error_nis_makefile}" \
    "deb=$debug" "deblil=$debug_lil" "mapname=${LPM_NIS_MAPNAME}" "pid=$$" \
    "gran=${s390_linux_2_gran}" ${YPDIR}/Makefile > ${YPDIR}/Makefile.new

MAKE_TARGET="target"

######################################################################
# This is all POST phase code, common on all platforms.
######################################################################
rc=$?
if [ $rc -eq 0 ]; then
    rc=`cat /tmp/cfg.out.$$`
fi
rm /tmp/cfg.out.$$

if [ $rc -eq 0 ]; then
    rc=${pdolp_cfg_pam_ok}
else 
    exit ${rc}
fi

#
# copy over the new makefile, and do an initial make
#
cp ${YPDIR}/Makefile.new ${YPDIR}/Makefile
cd ${YPDIR}
make ${MAKE_TARGET}
if [ $? -ne 0 ]; then
    exit ${pdolp_cfg_error_nis_make}
fi
if [ -f /tmp/${LPM_NIS_MAPNAME}.in ]; then
    rm /tmp/${LPM_NIS_MAPNAME}.in
fi

exit $rc
######################################################################
# END POST phase code.
######################################################################
